﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

public static class PythonUtils
{
    // Is there a constraint that restricts my generic method to numeric types?
    // https://stackoverflow.com/questions/32664/is-there-a-constraint-that-restricts-my-generic-method-to-numeric-types
    public static IEnumerable<T> Range<T>(T start, T stop, T step)
    {
        dynamic dStart = start;
        dynamic dStop = stop;
        dynamic dStep = step;

        if (dStep == 0)
            throw new ArgumentException("Parameter step cannot equal zero.");

        if (dStart < dStop && dStep > 0)
        {
            for (var i = dStart; i < dStop; i += dStep)
            {
                yield return i;
            }
        }
        else if (dStart > dStop && dStep < 0)
        {
            for (var i = dStart; i > dStop; i += dStep)
            {
                yield return i;
            }
        }
    }

    public static Dictionary<string, List<T>> Load<T>(string filePath, char delimiter = ',', int skipRows = 1)
    {
        Dictionary<string, List<T>> data = new Dictionary<string, List<T>>();
        Dictionary<int, string> headerIndex = new Dictionary<int, string>();

        foreach (string line in File.ReadAllLines(filePath))
        {
            if (skipRows >= 1)
            {
                if (data.Count == 0)
                {
                    string[] headers = line.Split(delimiter);
                    AddHeader<T>(data, headerIndex, headers);
                }

                skipRows--;
                continue;
            }

            string[] values = line.Split(delimiter);
            AddData(data, headerIndex, values);
        }

        return data;
    }

    private static void AddData<T>(Dictionary<string, List<T>> data, Dictionary<int, string> headerIndex, string[] values)
    {
        for (int i = 0; i < values.Length; i ++)
        {
            T number = ParseData<T>(values[i]);
            data[headerIndex[i]].Add(number);
        }
    }

    private static T ParseData<T>(string value)
    {
        T defaultValue = default(T);
        
        switch (defaultValue)
        {
            case short _:
                return (T)(object)short.Parse(value);

            case int _:
                return (T)(object)int.Parse(value);

            case double _:
                return (T)(object)double.Parse(value);

            default:
                throw new NotSupportedException(typeof(T).FullName);
        }
    }

    static void AddHeader<T>(Dictionary<string, List<T>> data, Dictionary<int, string> headerIndex, string[] headers)
    {
        int index = 0;

        foreach (string header in headers)
        {
            if (string.IsNullOrEmpty(header) == false)
            {
                headerIndex[index ++] = header;
                data[header] = new List<T>();
            }
        }
    }
}